@@ -63,6 +63,9 @@ from bzrlib.commands import (
 from bzrlib.version_info_formats.format_rio import (
     RioVersionInfoBuilder,
     )
+from bzrlib.send import (
+    format_registry as send_format_registry,
+    )
 
 
 if getattr(sys, "frozen", None):
@@ -78,22 +81,32 @@ def lazy_check_versions():
     try:
         from dulwich import __version__ as dulwich_version
     except ImportError:
-        raise ImportError("bzr-git: Please install dulwich, https://launchpad.net/dulwich")
+        raise bzr_errors.DependencyNotPresent("dulwich",
+            "bzr-git: Please install dulwich, https://launchpad.net/dulwich")
     else:
         if dulwich_version < dulwich_minimum_version:
-            raise ImportError("bzr-git: Dulwich is too old; at least %d.%d.%d is required" % dulwich_minimum_version)
+            raise bzr_errors.DependencyNotPresent("dulwich", "bzr-git: Dulwich is too old; at least %d.%d.%d is required" % dulwich_minimum_version)
 
-bzrdir.format_registry.register_lazy('git', 
+bzrdir.format_registry.register_lazy('git',
     "bzrlib.plugins.git.dir", "LocalGitBzrDirFormat",
     help='GIT repository.', native=False, experimental=True,
     )
 
 from bzrlib.revisionspec import revspec_registry
-revspec_registry.register_lazy("git:", "bzrlib.plugins.git.revspec", 
+revspec_registry.register_lazy("git:", "bzrlib.plugins.git.revspec",
     "RevisionSpec_git")
 
+try:
+    from bzrlib.revisionspec import dwim_revspecs
+except ImportError:
+    pass
+else:
+    from bzrlib.plugins.git.revspec import RevisionSpec_git
+    dwim_revspecs.append(RevisionSpec_git)
+
 
 class GitBzrDirFormat(bzrdir.BzrDirFormat):
+
     _lock_class = TransportLock
 
     def is_supported(self):
@@ -114,6 +127,7 @@ class LocalGitBzrDirFormat(GitBzrDirFormat):
         """Open this directory.
 
         """
+        lazy_check_versions()
         import dulwich
         # we dont grok readonly - git isn't integrated with transport.
         url = transport.base
@@ -136,11 +150,12 @@ class LocalGitBzrDirFormat(GitBzrDirFormat):
         if not isinstance(transport, LocalTransport):
             raise bzr_errors.NotBranchError(path=transport.base)
 
-        # This should quickly filter out most things that are not 
+        # This should quickly filter out most things that are not
         # git repositories, saving us the trouble from loading dulwich.
         if not transport.has(".git") and not transport.has("objects"):
             raise bzr_errors.NotBranchError(path=transport.base)
 
+        lazy_check_versions()
         import dulwich
         format = klass()
         try:
@@ -160,12 +175,12 @@ class LocalGitBzrDirFormat(GitBzrDirFormat):
         from bzrlib.transport.local import LocalTransport
 
         if not isinstance(transport, LocalTransport):
-            raise NotImplementedError(self.initialize, 
+            raise NotImplementedError(self.initialize,
                 "Can't create Git Repositories/branches on "
                 "non-local transports")
-
+        lazy_check_versions()
         from dulwich.repo import Repo
-        Repo.create(transport.local_abspath(".").encode(osutils._fs_enc)) 
+        Repo.create(transport.local_abspath(".").encode(osutils._fs_enc))
         return self.open(transport)
 
     def is_supported(self):
@@ -187,8 +202,7 @@ class RemoteGitBzrDirFormat(GitBzrDirFormat):
         url = transport.base
         if url.startswith('readonly+'):
             url = url[len('readonly+'):]
-        if (not url.startswith("git://") and 
-            not url.startswith("git+")):
+        if (not url.startswith("git://") and not url.startswith("git+")):
             raise bzr_errors.NotBranchError(transport.base)
         from bzrlib.plugins.git.remote import RemoteGitDir, GitSmartTransport
         if not isinstance(transport, GitSmartTransport):
@@ -203,8 +217,7 @@ class RemoteGitBzrDirFormat(GitBzrDirFormat):
         url = transport.base
         if url.startswith('readonly+'):
             url = url[len('readonly+'):]
-        if (not url.startswith("git://") and 
-            not url.startswith("git+")):
+        if (not url.startswith("git://") and not url.startswith("git+")):
             raise bzr_errors.NotBranchError(transport.base)
         # little ugly, but works
         format = klass()
@@ -226,9 +239,9 @@ class RemoteGitBzrDirFormat(GitBzrDirFormat):
 bzrdir.BzrDirFormat.register_control_format(LocalGitBzrDirFormat)
 bzrdir.BzrDirFormat.register_control_format(RemoteGitBzrDirFormat)
 
-register_transport_proto('git://', 
+register_transport_proto('git://',
         help="Access using the Git smart server protocol.")
-register_transport_proto('git+ssh://', 
+register_transport_proto('git+ssh://',
         help="Access using the Git smart server protocol over SSH.")
 
 register_lazy_transport("git://", 'bzrlib.plugins.git.remote',
@@ -236,11 +249,11 @@ register_lazy_transport("git://", 'bzrlib.plugins.git.remote',
 register_lazy_transport("git+ssh://", 'bzrlib.plugins.git.remote',
                         'SSHGitSmartTransport')
 
-foreign_vcs_registry.register_lazy("git", 
+foreign_vcs_registry.register_lazy("git",
     "bzrlib.plugins.git.mapping", "foreign_git", "Stupid content tracker")
 
 plugin_cmds.register_lazy("cmd_git_import", [], "bzrlib.plugins.git.commands")
-plugin_cmds.register_lazy("cmd_git_object", ["git-objects", "git-cat"], 
+plugin_cmds.register_lazy("cmd_git_object", ["git-objects", "git-cat"],
     "bzrlib.plugins.git.commands")
 
 def update_stanza(rev, stanza):
@@ -254,19 +267,15 @@ if rio_hooks is not None:
     rio_hooks.install_named_hook('revision', update_stanza, None)
 
 
-try:
-    from bzrlib.transport import transport_server_registry
-except ImportError:
-    pass
-else:
-    transport_server_registry.register_lazy('git',
-        'bzrlib.plugins.git.server', 
-        'serve_git',
-        'Git Smart server protocol over TCP. (default port: 9418)')
+from bzrlib.transport import transport_server_registry
+transport_server_registry.register_lazy('git',
+    'bzrlib.plugins.git.server',
+    'serve_git',
+    'Git Smart server protocol over TCP. (default port: 9418)')
 
 
 from bzrlib.repository import network_format_registry as repository_network_format_registry
-repository_network_format_registry.register_lazy('git', 
+repository_network_format_registry.register_lazy('git',
     'bzrlib.plugins.git.repository', 'GitRepositoryFormat')
 
 from bzrlib.bzrdir import network_format_registry as bzrdir_network_format_registry
@@ -279,6 +288,9 @@ def get_rich_root_format(stacked=False):
     else:
         return bzrdir.format_registry.make_bzrdir("default-rich-root")
 
+send_format_registry.register_lazy('git', 'bzrlib.plugins.git.send',
+                                   'send_git', 'Git am-style diff format')
+
 def test_suite():
     from bzrlib.plugins.git import tests
     return tests.test_suite()
