@@ -51,21 +51,15 @@ from bzrlib.plugins.git.errors import (
     NoSuchRef,
     )
 
-try:
-    from bzrlib.foreign import ForeignBranch
-except ImportError:
-    class ForeignBranch(branch.Branch):
-        def __init__(self, mapping):
-            self.mapping = mapping
-            super(ForeignBranch, self).__init__()
+from bzrlib.foreign import ForeignBranch
 
 
-def extract_tags(refs, mapping):
+def extract_tags(refs):
     ret = {}
     for k,v in refs.iteritems():
         if k.startswith("refs/tags/") and not k.endswith("^{}"):
             v = refs.get(k+"^{}", v)
-            ret[k[len("refs/tags/"):]] = mapping.revision_id_foreign_to_bzr(v)
+            ret[k[len("refs/tags/"):]] = v
     return ret
 
 
@@ -94,11 +88,15 @@ class LocalGitTagDict(tag.BasicTags):
 
     def get_tag_dict(self):
         ret = {}
-        for k,v in self.repository._git.refs.as_dict("refs/tags").iteritems():
-            obj = self.repository._git.get_object(v)
+        for k,v in extract_tags(self.repository._git.get_refs()).iteritems():
+            try:
+                obj = self.repository._git[v]
+            except KeyError:
+                mutter("Tag %s points at unknown object %s, ignoring", v, obj)
+                continue
             while isinstance(obj, Tag):
                 v = obj.object[1]
-                obj = self.repository._git.get_object(v)
+                obj = self.repository._git[v]
             if not isinstance(obj, Commit):
                 mutter("Tag %s points at object %r that is not a commit, "
                        "ignoring", k, obj)
@@ -111,14 +109,33 @@ class LocalGitTagDict(tag.BasicTags):
             self.branch.mapping.revision_id_bzr_to_foreign(revid)
 
 
+class DictTagDict(LocalGitTagDict):
+
+
+    def __init__(self, branch, tags):
+        super(DictTagDict, self).__init__(branch)
+        self._tags = tags
+
+    def get_tag_dict(self):
+        return self._tags
+
+
+
 class GitBranchFormat(branch.BranchFormat):
 
     def get_format_description(self):
         return 'Git Branch'
 
+    def network_name(self):
+        return "git"
+
     def supports_tags(self):
         return True
 
+    def get_foreign_tests_branch_factory(self):
+        from bzrlib.plugins.git.tests.test_branch import ForeignTestsBranchFactory
+        return ForeignTestsBranchFactory()
+
     def make_tags(self, branch):
         if getattr(branch.repository, "get_refs", None) is not None:
             from bzrlib.plugins.git.remote import RemoteGitTagDict
@@ -130,15 +147,30 @@ class GitBranchFormat(branch.BranchFormat):
 class GitBranch(ForeignBranch):
     """An adapter to git repositories for bzr Branch objects."""
 
-    def __init__(self, bzrdir, repository, name, lockfiles):
+    def __init__(self, bzrdir, repository, name, lockfiles, tagsdict=None):
         self.repository = repository
         self._format = GitBranchFormat()
         self.control_files = lockfiles
         self.bzrdir = bzrdir
         super(GitBranch, self).__init__(repository.get_mapping())
+        if tagsdict is not None:
+            self.tags = DictTagDict(self, tagsdict)
         self.name = name
         self._head = None
-        self.base = bzrdir.transport.base
+        self.base = bzrdir.root_transport.base
+
+    def _get_checkout_format(self):
+        """Return the most suitable metadir for a checkout of this branch.
+        Weaves are used if this branch's repository uses weaves.
+        """
+        return get_rich_root_format()
+
+    def get_child_submit_format(self):
+        """Return the preferred format of submissions to this branch."""
+        ret = self.get_config().get_user_option("child_submit_format")
+        if ret is not None:
+            return ret
+        return "git"
 
     def _get_nick(self, local=False, possible_master_transports=None):
         """Find the nick name for this branch.
@@ -165,7 +197,7 @@ class GitBranch(ForeignBranch):
 
     def get_stacked_on_url(self):
         # Git doesn't do stacking (yet...)
-        return None
+        raise errors.UnstackableBranchFormat(self._format, self.base)
 
     def get_parent(self):
         """See Branch.get_parent()."""
@@ -199,18 +231,10 @@ class GitBranch(ForeignBranch):
         return branch.InterBranch.get(self, target)._basic_push(
             overwrite, stop_revision)
 
- 
+
 class LocalGitBranch(GitBranch):
     """A local Git branch."""
 
-    def _get_checkout_format(self):
-        """Return the most suitable metadir for a checkout of this branch.
-        Weaves are used if this branch's repository uses weaves.
-        """
-        format = self.repository.bzrdir.checkout_metadir()
-        format.set_branch_format(self._format)
-        return format
-
     def create_checkout(self, to_location, revision_id=None, lightweight=False,
         accelerator_tree=None, hardlink=False):
         if lightweight:
@@ -218,7 +242,7 @@ class LocalGitBranch(GitBranch):
             t.ensure_base()
             format = self._get_checkout_format()
             checkout = format.initialize_on_transport(t)
-            from_branch = branch.BranchReferenceFormat().initialize(checkout, 
+            from_branch = branch.BranchReferenceFormat().initialize(checkout,
                 self)
             tree = checkout.create_workingtree(revision_id,
                 from_branch=from_branch, hardlink=hardlink)
@@ -227,7 +251,7 @@ class LocalGitBranch(GitBranch):
             return self._create_heavyweight_checkout(to_location, revision_id,
             hardlink)
 
-    def _create_heavyweight_checkout(self, to_location, revision_id=None, 
+    def _create_heavyweight_checkout(self, to_location, revision_id=None,
                                      hardlink=False):
         """Create a new heavyweight checkout of this branch.
 
@@ -240,7 +264,7 @@ class LocalGitBranch(GitBranch):
             to_location, force_new_tree=False, format=get_rich_root_format())
         checkout = checkout_branch.bzrdir
         checkout_branch.bind(self)
-        # pull up to the specified revision_id to set the initial 
+        # pull up to the specified revision_id to set the initial
         # branch tip correctly, and seed it with history.
         checkout_branch.pull(self, stop_revision=revision_id)
         return checkout.create_workingtree(revision_id, hardlink=hardlink)
@@ -298,7 +322,7 @@ class GitBranchPullResult(branch.PullResult):
             if self.old_revid == self.new_revid:
                 to_file.write('No revisions to pull.\n')
             else:
-                to_file.write('Now on revision %d (git sha: %s).\n' % 
+                to_file.write('Now on revision %d (git sha: %s).\n' %
                         (self.new_revno, self.new_git_head))
         self._show_tag_conficts(to_file)
 
@@ -327,15 +351,20 @@ class InterFromGitBranch(branch.GenericInterBranch):
     """InterBranch implementation that pulls from Git into bzr."""
 
     @classmethod
-    def is_compatible(self, source, target):
-        return (isinstance(source, GitBranch) and 
-                not isinstance(target, GitBranch))
+    def _get_interrepo(self, source, target):
+        return repository.InterRepository.get(source.repository,
+            target.repository)
+
+    @classmethod
+    def is_compatible(cls, source, target):
+        return (isinstance(source, GitBranch) and
+                not isinstance(target, GitBranch) and
+                (getattr(cls._get_interrepo(source, target), "fetch_objects", None) is not None))
 
     def update_revisions(self, stop_revision=None, overwrite=False,
         graph=None):
         """See InterBranch.update_revisions()."""
-        interrepo = repository.InterRepository.get(self.source.repository, 
-            self.target.repository)
+        interrepo = self._get_interrepo(self.source, self.target)
         self._head = None
         self._last_revid = None
         def determine_wants(heads):
@@ -343,7 +372,7 @@ class InterFromGitBranch(branch.GenericInterBranch):
                 raise NoSuchRef(self.source.name, heads.keys())
             if stop_revision is not None:
                 self._last_revid = stop_revision
-                self._head, mapping = self.source.repository.lookup_git_revid(
+                self._head, mapping = self.source.repository.lookup_bzr_revision_id(
                     stop_revision)
             else:
                 self._head = heads[self.source.name]
@@ -388,7 +417,7 @@ class InterFromGitBranch(branch.GenericInterBranch):
             graph = self.target.repository.get_graph(self.source.repository)
             result.old_revno, result.old_revid = \
                 self.target.last_revision_info()
-            self.update_revisions(stop_revision, overwrite=overwrite, 
+            self.update_revisions(stop_revision, overwrite=overwrite,
                 graph=graph)
             result.new_git_head = self._head
             result.tag_conflicts = self.source.tags.merge_to(self.target.tags,
@@ -413,8 +442,7 @@ class InterFromGitBranch(branch.GenericInterBranch):
         result.target_branch = self.target
         graph = self.target.repository.get_graph(self.source.repository)
         result.old_revno, result.old_revid = self.target.last_revision_info()
-        self.update_revisions(stop_revision, overwrite=overwrite, 
-            graph=graph)
+        self.update_revisions(stop_revision, overwrite=overwrite, graph=graph)
         result.new_git_head = self._head
         result.tag_conflicts = self.source.tags.merge_to(self.target.tags,
             overwrite)
@@ -432,7 +460,7 @@ class InterGitLocalRemoteBranch(InterGitBranch):
     @classmethod
     def is_compatible(self, source, target):
         from bzrlib.plugins.git.remote import RemoteGitBranch
-        return (isinstance(source, LocalGitBranch) and 
+        return (isinstance(source, LocalGitBranch) and
                 isinstance(target, RemoteGitBranch))
 
     def _basic_push(self, overwrite=False, stop_revision=None):
@@ -444,12 +472,12 @@ class InterGitLocalRemoteBranch(InterGitBranch):
         # FIXME: Check for diverged branches
         def get_changed_refs(old_refs):
             result.old_revid = self.target.mapping.revision_id_foreign_to_bzr(old_refs.get("refs/heads/master", "0" * 40))
-            refs = { "refs/heads/master": self.source.repository.lookup_git_revid(stop_revision)[0] }
+            refs = { "refs/heads/master": self.source.repository.lookup_bzr_revision_id(stop_revision)[0] }
             result.new_revid = stop_revision
             for name, sha in self.source.repository._git.refs.as_dict("refs/tags").iteritems():
                 refs["refs/tags/%s" % name] = sha
             return refs
-        self.target.repository.send_pack(get_changed_refs, 
+        self.target.repository.send_pack(get_changed_refs,
                 self.source.repository._git.object_store.generate_pack_contents)
         return result
 
@@ -460,7 +488,7 @@ class InterGitRemoteLocalBranch(InterGitBranch):
     @classmethod
     def is_compatible(self, source, target):
         from bzrlib.plugins.git.remote import RemoteGitBranch
-        return (isinstance(source, RemoteGitBranch) and 
+        return (isinstance(source, RemoteGitBranch) and
                 isinstance(target, LocalGitBranch))
 
     def _basic_push(self, overwrite=False, stop_revision=None):
@@ -475,11 +503,12 @@ class InterGitRemoteLocalBranch(InterGitBranch):
         return result
 
     def update_tags(self, refs):
-        for name, revid in extract_tags(refs, self.target.mapping).iteritems():
+        for name, v in extract_tags(refs).iteritems():
+            revid = self.target.mapping.revision_id_foreign_to_bzr(v)
             self.target.tags.set_tag(name, revid)
 
     def update_refs(self, stop_revision=None):
-        interrepo = repository.InterRepository.get(self.source.repository, 
+        interrepo = repository.InterRepository.get(self.source.repository,
             self.target.repository)
         if stop_revision is None:
             refs = interrepo.fetch_refs(branches=["HEAD"])
@@ -488,7 +517,7 @@ class InterGitRemoteLocalBranch(InterGitBranch):
             refs = interrepo.fetch_refs(revision_id=stop_revision)
         return refs, stop_revision
 
-    def pull(self, stop_revision=None, overwrite=False, 
+    def pull(self, stop_revision=None, overwrite=False,
         possible_transports=None, local=False):
         # This type of branch can't be bound.
         if local:
@@ -503,19 +532,23 @@ class InterGitRemoteLocalBranch(InterGitBranch):
         result.new_revid = self.target.last_revision()
         return result
 
-    
+
 class InterToGitBranch(branch.InterBranch):
     """InterBranch implementation that pulls from Git into bzr."""
 
+    @staticmethod
+    def _get_branch_formats_to_test():
+        return None, None
+
     @classmethod
     def is_compatible(self, source, target):
-        return (not isinstance(source, GitBranch) and 
+        return (not isinstance(source, GitBranch) and
                 isinstance(target, GitBranch))
 
     def update_revisions(self, *args, **kwargs):
         raise NoPushSupport()
 
-    def push(self, overwrite=True, stop_revision=None, 
+    def push(self, overwrite=True, stop_revision=None,
              _override_hook_source_branch=None):
         raise NoPushSupport()
 
@@ -523,7 +556,10 @@ class InterToGitBranch(branch.InterBranch):
         result = GitBranchPushResult()
         result.source_branch = self.source
         result.target_branch = self.target
-        result.old_revid = self.target.last_revision()
+        try:
+            result.old_revid = self.target.last_revision()
+        except NoSuchRef:
+            result.old_revid = revision.NULL_REVISION
         if stop_revision is None:
             stop_revision = self.source.last_revision()
         # FIXME: Check for diverged branches
