@@ -27,12 +27,14 @@ import stat
 
 from bzrlib import (
     debug,
+    lru_cache,
     osutils,
     trace,
     ui,
     urlutils,
     )
 from bzrlib.errors import (
+    BzrError,
     NoSuchId,
     )
 from bzrlib.inventory import (
@@ -40,9 +42,7 @@ from bzrlib.inventory import (
     InventoryDirectory,
     InventoryFile,
     InventoryLink,
-    )
-from bzrlib.lru_cache import (
-    LRUCache,
+    TreeReference,
     )
 from bzrlib.repository import (
     InterRepository,
@@ -71,13 +71,16 @@ from bzrlib.plugins.git.remote import (
     RemoteGitRepository,
     )
 from bzrlib.plugins.git.repository import (
-    GitRepository, 
+    GitRepository,
     GitRepositoryFormat,
     LocalGitRepository,
     )
 
 
-def import_git_blob(texts, mapping, path, hexsha, base_inv, base_ie, parent_id, 
+MAX_INV_CACHE_SIZE = 50 * 1024 * 1024
+
+
+def import_git_blob(texts, mapping, path, hexsha, base_inv, base_ie, parent_id,
     revision_id, parent_invs, shagitmap, lookup_object, executable, symlink):
     """Import a git blob object into a bzr repository.
 
@@ -152,7 +155,7 @@ def import_git_blob(texts, mapping, path, hexsha, base_inv, base_ie, parent_id,
     else:
         shamap = []
     invdelta = []
-    if base_ie is not None: 
+    if base_ie is not None:
         old_path = base_inv.id2path(file_id)
         if base_ie.kind == "directory":
             invdelta.extend(remove_disappeared_children(old_path, base_ie.children, []))
@@ -162,9 +165,28 @@ def import_git_blob(texts, mapping, path, hexsha, base_inv, base_ie, parent_id,
     return (invdelta, shamap)
 
 
-def import_git_submodule(texts, mapping, path, hexsha, base_inv, base_ie, parent_id, 
-    revision_id, parent_invs, shagitmap, lookup_object):
-    raise NotImplementedError(import_git_submodule)
+class SubmodulesRequireSubtrees(BzrError):
+    _fmt = """The repository you are fetching from contains submodules. To continue, upgrade your Bazaar repository to a format that supports nested trees, such as 'development-subtree'."""
+    internal = False
+
+
+def import_git_submodule(texts, mapping, path, hexsha, base_inv, base_ie,
+    parent_id, revision_id, parent_invs, shagitmap, lookup_object):
+    file_id = mapping.generate_file_id(path)
+    ie = TreeReference(file_id, urlutils.basename(path.decode("utf-8")),
+        parent_id)
+    ie.revision = revision_id
+    if base_ie is None:
+        oldpath = None
+    else:
+        oldpath = path
+        if (base_ie.kind == ie.kind and
+            base_ie.reference_revision == ie.reference_revision):
+            ie.revision = base_ie.revision
+    ie.reference_revision = mapping.revision_id_foreign_to_bzr(hexsha)
+    texts.insert_record_stream([FulltextContentFactory((file_id, ie.revision), (), None, "")])
+    invdelta = [(oldpath, path, file_id, ie)]
+    return invdelta, {}, {}
 
 
 def remove_disappeared_children(path, base_children, existing_children):
@@ -179,8 +201,8 @@ def remove_disappeared_children(path, base_children, existing_children):
     return ret
 
 
-def import_git_tree(texts, mapping, path, hexsha, base_inv, base_ie, parent_id, 
-    revision_id, parent_invs, shagitmap, lookup_object):
+def import_git_tree(texts, mapping, path, hexsha, base_inv, base_ie, parent_id,
+    revision_id, parent_invs, shagitmap, lookup_object, allow_submodules=False):
     """Import a git tree object into a bzr repository.
 
     :param texts: VersionedFiles object to add to
@@ -192,7 +214,7 @@ def import_git_tree(texts, mapping, path, hexsha, base_inv, base_ie, parent_id,
     invdelta = []
     file_id = mapping.generate_file_id(path)
     # We just have to hope this is indeed utf-8:
-    ie = InventoryDirectory(file_id, urlutils.basename(path.decode("utf-8")), 
+    ie = InventoryDirectory(file_id, urlutils.basename(path.decode("utf-8")),
         parent_id)
     if base_ie is None:
         # Newly appeared here
@@ -228,13 +250,16 @@ def import_git_tree(texts, mapping, path, hexsha, base_inv, base_ie, parent_id,
         child_path = osutils.pathjoin(path, name)
         if stat.S_ISDIR(mode):
             subinvdelta, grandchildmodes, subshamap = import_git_tree(
-                    texts, mapping, child_path, child_hexsha, base_inv, 
-                    base_children.get(basename), file_id, revision_id, parent_invs, shagitmap,
-                    lookup_object)
+                    texts, mapping, child_path, child_hexsha, base_inv,
+                    base_children.get(basename), file_id, revision_id,
+                    parent_invs, shagitmap, lookup_object,
+                    allow_submodules=allow_submodules)
             invdelta.extend(subinvdelta)
             child_modes.update(grandchildmodes)
             shamap.extend(subshamap)
         elif S_ISGITLINK(mode): # submodule
+            if not allow_submodules:
+                raise SubmodulesRequireSubtrees()
             subinvdelta, grandchildmodes, subshamap = import_git_submodule(
                     texts, mapping, child_path, child_hexsha, base_inv, base_children.get(basename),
                     file_id, revision_id, parent_invs, shagitmap, lookup_object)
@@ -242,9 +267,9 @@ def import_git_tree(texts, mapping, path, hexsha, base_inv, base_ie, parent_id,
             child_modes.update(grandchildmodes)
             shamap.extend(subshamap)
         else:
-            subinvdelta, subshamap = import_git_blob(texts, mapping, 
+            subinvdelta, subshamap = import_git_blob(texts, mapping,
                     child_path, child_hexsha, base_inv, base_children.get(basename), file_id,
-                    revision_id, parent_invs, shagitmap, lookup_object, 
+                    revision_id, parent_invs, shagitmap, lookup_object,
                     mode_is_executable(mode), stat.S_ISLNK(mode))
             invdelta.extend(subinvdelta)
             shamap.extend(subshamap)
@@ -253,13 +278,70 @@ def import_git_tree(texts, mapping, path, hexsha, base_inv, base_ie, parent_id,
             child_modes[child_path] = mode
     # Remove any children that have disappeared
     if base_ie is not None and base_ie.kind == "directory":
-        invdelta.extend(remove_disappeared_children(base_inv.id2path(file_id), 
+        invdelta.extend(remove_disappeared_children(base_inv.id2path(file_id),
             base_children, existing_children))
     shamap.append((hexsha, "tree", (file_id, revision_id)))
     return invdelta, child_modes, shamap
 
 
-def import_git_objects(repo, mapping, object_iter, target_git_object_retriever, 
+def approx_inv_size(inv):
+    # Very rough estimate, 1k per inventory entry
+    return len(inv) * 1024
+
+
+def import_git_commit(repo, mapping, head, lookup_object,
+                      target_git_object_retriever, parent_invs_cache):
+    o = lookup_object(head)
+    rev = mapping.import_commit(o)
+    # We have to do this here, since we have to walk the tree and
+    # we need to make sure to import the blobs / trees with the right
+    # path; this may involve adding them more than once.
+    parent_invs = []
+    for parent_id in rev.parent_ids:
+        try:
+            parent_invs.append(parent_invs_cache[parent_id])
+        except KeyError:
+            parent_inv = repo.get_inventory(parent_id)
+            parent_invs.append(parent_inv)
+            parent_invs_cache[parent_id] = parent_inv
+    if parent_invs == []:
+        base_inv = Inventory(root_id=None)
+        base_ie = None
+    else:
+        base_inv = parent_invs[0]
+        base_ie = base_inv.root
+    inv_delta, unusual_modes, shamap = import_git_tree(repo.texts,
+            mapping, "", o.tree, base_inv, base_ie, None, rev.revision_id,
+            parent_invs, target_git_object_retriever._idmap, lookup_object,
+            allow_submodules=getattr(repo._format, "supports_tree_reference", False))
+    target_git_object_retriever._idmap.add_entries(shamap)
+    if unusual_modes != {}:
+        for path, mode in unusual_modes.iteritems():
+            warn_unusual_mode(rev.foreign_revid, path, mode)
+        mapping.import_unusual_file_modes(rev, unusual_modes)
+    try:
+        basis_id = rev.parent_ids[0]
+    except IndexError:
+        basis_id = NULL_REVISION
+        base_inv = None
+    rev.inventory_sha1, inv = repo.add_inventory_by_delta(basis_id,
+              inv_delta, rev.revision_id, rev.parent_ids,
+              base_inv)
+    parent_invs_cache[rev.revision_id] = inv
+    repo.add_revision(rev.revision_id, rev)
+    if "verify" in debug.debug_flags:
+        new_unusual_modes = mapping.export_unusual_file_modes(rev)
+        if new_unusual_modes != unusual_modes:
+            raise AssertionError("unusual modes don't match: %r != %r" % (unusual_modes, new_unusual_modes))
+        objs = inventory_to_tree_and_blobs(inv, repo.texts, mapping, unusual_modes)
+        for sha1, newobj, path in objs:
+            assert path is not None
+            oldobj = tree_lookup_path(lookup_object, o.tree, path)
+            if oldobj != newobj:
+                raise AssertionError("%r != %r in %s" % (oldobj, newobj, path))
+
+
+def import_git_objects(repo, mapping, object_iter, target_git_object_retriever,
         heads, pb=None):
     """Import a set of git objects into a bzr repository.
 
@@ -267,6 +349,7 @@ def import_git_objects(repo, mapping, object_iter, target_git_object_retriever,
     :param mapping: Mapping to use
     :param object_iter: Iterator over Git objects.
     """
+    target_git_object_retriever._idmap.start_write_group() # FIXME: try/finally
     def lookup_object(sha):
         try:
             return object_iter[sha]
@@ -274,11 +357,10 @@ def import_git_objects(repo, mapping, object_iter, target_git_object_retriever,
             return target_git_object_retriever[sha]
     # TODO: a more (memory-)efficient implementation of this
     graph = []
-    root_trees = {}
-    revisions = {}
     checked = set()
     heads = list(heads)
-    parent_invs_cache = LRUCache(50)
+    parent_invs_cache = lru_cache.LRUSizeCache(compute_size=approx_inv_size,
+                                               max_size=MAX_INV_CACHE_SIZE)
     # Find and convert commit objects
     while heads:
         if pb is not None:
@@ -295,68 +377,39 @@ def import_git_objects(repo, mapping, object_iter, target_git_object_retriever,
             if repo.has_revision(rev.revision_id):
                 continue
             squash_revision(repo, rev)
-            root_trees[rev.revision_id] = o.tree
-            revisions[rev.revision_id] = rev
-            graph.append((rev.revision_id, rev.parent_ids))
-            target_git_object_retriever._idmap.add_entry(o.id, "commit", 
+            graph.append((o.id, o.parents))
+            target_git_object_retriever._idmap.add_entry(o.id, "commit",
                     (rev.revision_id, o.tree))
             heads.extend([p for p in o.parents if p not in checked])
         elif isinstance(o, Tag):
             heads.append(o.object[1])
         else:
             trace.warning("Unable to import head object %r" % o)
-        checked.add(head)
+        checked.add(o.id)
+    del checked
     # Order the revisions
     # Create the inventory objects
-    for i, revid in enumerate(topo_sort(graph)):
-        if pb is not None:
-            pb.update("fetching revisions", i, len(graph))
-        rev = revisions[revid]
-        # We have to do this here, since we have to walk the tree and 
-        # we need to make sure to import the blobs / trees with the right 
-        # path; this may involve adding them more than once.
-        parent_invs = []
-        for parent_id in rev.parent_ids:
-            try:
-                parent_invs.append(parent_invs_cache[parent_id])
-            except KeyError:
-                parent_inv = repo.get_inventory(parent_id)
-                parent_invs.append(parent_inv)
-                parent_invs_cache[parent_id] = parent_inv
-        if parent_invs == []:
-            base_inv = Inventory(root_id=None)
-            base_ie = None
-        else:
-            base_inv = parent_invs[0]
-            base_ie = base_inv.root
-        inv_delta, unusual_modes, shamap = import_git_tree(repo.texts, 
-                mapping, "", root_trees[revid], base_inv, base_ie, None, revid, 
-                parent_invs, target_git_object_retriever._idmap, lookup_object)
-        target_git_object_retriever._idmap.add_entries(shamap)
-        if unusual_modes != {}:
-            for path, mode in unusual_modes.iteritems():
-                warn_unusual_mode(rev.foreign_revid, path, mode)
-            mapping.import_unusual_file_modes(rev, unusual_modes)
+    batch_size = 100
+    revision_ids = topo_sort(graph)
+    pack_hints = []
+    for offset in range(0, len(revision_ids), batch_size):
+        repo.start_write_group()
         try:
-            basis_id = rev.parent_ids[0]
-        except IndexError:
-            basis_id = NULL_REVISION
-        rev.inventory_sha1, inv = repo.add_inventory_by_delta(basis_id,
-                  inv_delta, rev.revision_id, rev.parent_ids)
-        parent_invs_cache[rev.revision_id] = inv
-        repo.add_revision(rev.revision_id, rev)
-        if "verify" in debug.debug_flags:
-            new_unusual_modes = mapping.export_unusual_file_modes(rev)
-            if new_unusual_modes != unusual_modes:
-                raise AssertionError("unusual modes don't match: %r != %r" % (unusual_modes, new_unusual_modes))
-            objs = inventory_to_tree_and_blobs(inv, repo.texts, mapping, unusual_modes)
-            for sha1, newobj, path in objs:
-                assert path is not None
-                oldobj = tree_lookup_path(lookup_object, root_trees[revid], path)
-                if oldobj != newobj:
-                    raise AssertionError("%r != %r in %s" % (oldobj, newobj, path))
-
-    target_git_object_retriever._idmap.commit()
+            for i, head in enumerate(revision_ids[offset:offset+batch_size]):
+                if pb is not None:
+                    pb.update("fetching revisions", offset+i, len(revision_ids))
+                import_git_commit(repo, mapping, head, lookup_object,
+                                  target_git_object_retriever,
+                                  parent_invs_cache)
+        except:
+            repo.abort_write_group()
+            raise
+        else:
+            hint = repo.commit_write_group()
+            if hint is not None:
+                pack_hints.extend(hint)
+    target_git_object_retriever._idmap.commit_write_group()
+    return pack_hints
 
 
 class InterGitRepository(InterRepository):
@@ -378,10 +431,10 @@ class InterGitRepository(InterRepository):
 
 
 class InterGitNonGitRepository(InterGitRepository):
-    """Base InterRepository that copies revisions from a Git into a non-Git 
+    """Base InterRepository that copies revisions from a Git into a non-Git
     repository."""
 
-    def fetch_refs(self, revision_id=None, pb=None, find_ghosts=False, 
+    def fetch_refs(self, revision_id=None, pb=None, find_ghosts=False,
               mapping=None, fetch_spec=None):
         if mapping is None:
             mapping = self.source.get_mapping()
@@ -422,7 +475,7 @@ def report_git_progress(pb, text):
 
 
 class InterRemoteGitNonGitRepository(InterGitNonGitRepository):
-    """InterRepository that copies revisions from a remote Git into a non-Git 
+    """InterRepository that copies revisions from a remote Git into a non-Git
     repository."""
 
     def get_target_heads(self):
@@ -448,21 +501,16 @@ class InterRemoteGitNonGitRepository(InterGitNonGitRepository):
                 wants = determine_wants(heads)
                 recorded_wants.extend(wants)
                 return wants
-        
+
             create_pb = None
             if pb is None:
                 create_pb = pb = ui.ui_factory.nested_progress_bar()
             try:
-                self.target.start_write_group()
-                try:
-                    objects_iter = self.source.fetch_objects(
-                                record_determine_wants, graph_walker, 
-                                store.get_raw, progress)
-                    import_git_objects(self.target, mapping, objects_iter, 
-                            store, recorded_wants, pb)
-                finally:
-                    pack_hint = self.target.commit_write_group()
-                return pack_hint
+                objects_iter = self.source.fetch_objects(
+                            record_determine_wants, graph_walker,
+                            store.get_raw, progress)
+                return import_git_objects(self.target, mapping,
+                    objects_iter, store, recorded_wants, pb)
             finally:
                 if create_pb:
                     create_pb.finished()
@@ -473,13 +521,13 @@ class InterRemoteGitNonGitRepository(InterGitNonGitRepository):
     def is_compatible(source, target):
         """Be compatible with GitRepository."""
         # FIXME: Also check target uses VersionedFile
-        return (isinstance(source, RemoteGitRepository) and 
+        return (isinstance(source, RemoteGitRepository) and
                 target.supports_rich_root() and
                 not isinstance(target, GitRepository))
 
 
 class InterLocalGitNonGitRepository(InterGitNonGitRepository):
-    """InterRepository that copies revisions from a local Git into a non-Git 
+    """InterRepository that copies revisions from a local Git into a non-Git
     repository."""
 
     def fetch_objects(self, determine_wants, mapping, pb=None):
@@ -491,14 +539,9 @@ class InterLocalGitNonGitRepository(InterGitNonGitRepository):
         try:
             self.target.lock_write()
             try:
-                self.target.start_write_group()
-                try:
-                    import_git_objects(self.target, mapping, 
-                            self.source._git.object_store, 
-                            target_git_object_retriever, wants, pb)
-                finally:
-                    pack_hint = self.target.commit_write_group()
-                return pack_hint
+                return import_git_objects(self.target, mapping,
+                    self.source._git.object_store, target_git_object_retriever,
+                    wants, pb)
             finally:
                 self.target.unlock()
         finally:
@@ -509,7 +552,7 @@ class InterLocalGitNonGitRepository(InterGitNonGitRepository):
     def is_compatible(source, target):
         """Be compatible with GitRepository."""
         # FIXME: Also check target uses VersionedFile
-        return (isinstance(source, LocalGitRepository) and 
+        return (isinstance(source, LocalGitRepository) and
                 target.supports_rich_root() and
                 not isinstance(target, GitRepository))
 
@@ -517,12 +560,35 @@ class InterLocalGitNonGitRepository(InterGitNonGitRepository):
 class InterGitGitRepository(InterGitRepository):
     """InterRepository that copies between Git repositories."""
 
-    def fetch_refs(self, revision_id=None, pb=None, find_ghosts=False, 
+    def fetch_objects(self, determine_wants, mapping, pb=None):
+        def progress(text):
+            trace.note("git: %s", text)
+        graphwalker = self.target._git.get_graph_walker()
+        if (isinstance(self.source, LocalGitRepository) and
+            isinstance(self.target, LocalGitRepository)):
+            return self.source._git.fetch(self.target._git, determine_wants,
+                progress)
+        elif (isinstance(self.source, LocalGitRepository) and
+              isinstance(self.target, RemoteGitRepository)):
+            raise NotImplementedError
+        elif (isinstance(self.source, RemoteGitRepository) and
+              isinstance(self.target, LocalGitRepository)):
+            f, commit = self.target._git.object_store.add_thin_pack()
+            try:
+                refs = self.source._git.fetch_pack(determine_wants,
+                    graphwalker, f.write, progress)
+                commit()
+                return refs
+            except:
+                f.close()
+                raise
+        else:
+            raise AssertionError
+
+    def fetch_refs(self, revision_id=None, pb=None, find_ghosts=False,
               mapping=None, fetch_spec=None, branches=None):
         if mapping is None:
             mapping = self.source.get_mapping()
-        def progress(text):
-            trace.info("git: %s", text)
         r = self.target._git
         if revision_id is not None:
             args = [mapping.revision_id_bzr_to_foreign(revision_id)[0]]
@@ -534,20 +600,11 @@ class InterGitGitRepository(InterGitRepository):
             determine_wants = r.object_store.determine_wants_all
         else:
             determine_wants = lambda x: [y for y in args if not y in r.object_store]
+        return self.fetch_objects(determine_wants, mapping)
 
-        graphwalker = r.get_graph_walker()
-        f, commit = r.object_store.add_thin_pack()
-        try:
-            refs = self.source.fetch_pack(determine_wants, graphwalker,
-                                          f.write, progress)
-            commit()
-            return refs
-        except:
-            f.close()
-            raise
 
     @staticmethod
     def is_compatible(source, target):
         """Be compatible with GitRepository."""
-        return (isinstance(source, GitRepository) and 
+        return (isinstance(source, GitRepository) and
                 isinstance(target, GitRepository))
