@@ -16,6 +16,7 @@
 
 import bzrlib
 from bzrlib import (
+    config,
     tag,
     trace,
     ui,
@@ -64,6 +65,7 @@ from dulwich.errors import (
     )
 from dulwich.pack import (
     Pack,
+    PackData,
     )
 import os
 import tempfile
@@ -73,7 +75,7 @@ import urlparse
 from dulwich.pack import load_pack_index
 
 
-# Don't run any tests on GitSmartTransport as it is not intended to be 
+# Don't run any tests on GitSmartTransport as it is not intended to be
 # a full implementation of Transport
 def get_test_permutations():
     return []
@@ -84,7 +86,8 @@ class GitSmartTransport(Transport):
     def __init__(self, url, _client=None):
         Transport.__init__(self, url)
         (scheme, _, loc, _, _) = urlparse.urlsplit(url)
-        hostport, self._path = urllib.splithost(loc)
+        hostport, escaped_path = urllib.splithost(loc)
+        self._path = urllib.unquote(escaped_path)
         (self._username, hostport) = urllib.splituser(hostport)
         (self._host, self._port) = urllib.splitnport(hostport, None)
         self._client = _client
@@ -107,7 +110,7 @@ class GitSmartTransport(Transport):
                 info("git: %s" % text)
         client = self._get_client(thin_packs=False)
         try:
-            return client.fetch_pack(self._get_path(), determine_wants, 
+            return client.fetch_pack(self._get_path(), determine_wants,
                 graph_walker, pack_data, progress)
         except GitProtocolError, e:
             raise BzrError(e)
@@ -115,7 +118,7 @@ class GitSmartTransport(Transport):
     def send_pack(self, get_changed_refs, generate_pack_contents):
         client = self._get_client(thin_packs=False)
         try:
-            return client.send_pack(self._get_path(), get_changed_refs, 
+            return client.send_pack(self._get_path(), get_changed_refs,
                 generate_pack_contents)
         except GitProtocolError, e:
             raise BzrError(e)
@@ -184,7 +187,7 @@ class RemoteGitDir(GitDir):
         # TODO: Support for multiple branches in one bzrdir in bzrlib!
         return RemoteGitBranch(self, repo, "HEAD", self._lockfiles)
 
-    def open_workingtree(self):
+    def open_workingtree(self, recommend_upgrade=False):
         raise NotLocalUrl(self.transport.base)
 
 
@@ -200,6 +203,12 @@ class TemporaryPackIterator(Pack):
         super(TemporaryPackIterator, self).__init__(path)
         self.resolve_ext_ref = resolve_ext_ref
 
+    @property
+    def data(self):
+        if self._data is None:
+            self._data = PackData(self._data_path)
+        return self._data
+
     @property
     def index(self):
         if self._idx is None:
@@ -216,8 +225,12 @@ class TemporaryPackIterator(Pack):
         return self._idx
 
     def __del__(self):
-        os.remove(self._data_path)
-        os.remove(self._idx_path)
+        if self._idx is not None:
+            self._idx.close()
+            os.remove(self._idx_path)
+        if self._data is not None:
+            self._data.close()
+            os.remove(self._data_path)
 
 
 class RemoteGitRepository(GitRepository):
@@ -241,11 +254,11 @@ class RemoteGitRepository(GitRepository):
     def get_refs(self):
         if self._refs is not None:
             return self._refs
-        self._refs = self.bzrdir.root_transport.fetch_pack(lambda x: [], None, 
+        self._refs = self.bzrdir.root_transport.fetch_pack(lambda x: [], None,
             lambda x: None, lambda x: trace.mutter("git: %s" % x))
         return self._refs
 
-    def fetch_pack(self, determine_wants, graph_walker, pack_data, 
+    def fetch_pack(self, determine_wants, graph_walker, pack_data,
                    progress=None):
         return self._transport.fetch_pack(determine_wants, graph_walker,
                                           pack_data, progress)
@@ -253,15 +266,17 @@ class RemoteGitRepository(GitRepository):
     def send_pack(self, get_changed_refs, generate_pack_contents):
         return self._transport.send_pack(get_changed_refs, generate_pack_contents)
 
-    def fetch_objects(self, determine_wants, graph_walker, resolve_ext_ref, progress=None):
+    def fetch_objects(self, determine_wants, graph_walker, resolve_ext_ref,
+                      progress=None):
         fd, path = tempfile.mkstemp(suffix=".pack")
-        self.fetch_pack(determine_wants, graph_walker, lambda x: os.write(fd, x), progress)
+        self.fetch_pack(determine_wants, graph_walker,
+            lambda x: os.write(fd, x), progress)
         os.close(fd)
         if os.path.getsize(path) == 0:
             return EmptyObjectStoreIterator()
         return TemporaryPackIterator(path[:-len(".pack")], resolve_ext_ref)
 
-    def lookup_git_revid(self, bzr_revid):
+    def lookup_bzr_revision_id(self, bzr_revid):
         # This won't work for any round-tripped bzr revisions, but it's a start..
         try:
             return mapping_registry.revision_id_bzr_to_foreign(bzr_revid)
@@ -276,7 +291,10 @@ class RemoteGitTagDict(tag.BasicTags):
         self.repository = branch.repository
 
     def get_tag_dict(self):
-        return extract_tags(self.repository.get_refs(), self.branch.mapping)
+        tags = {}
+        for k, v in extract_tags(self.repository.get_refs()).iteritems():
+            tags[k] = self.branch.mapping.revision_id_foreign_to_bzr(v)
+        return tags
 
     def set_tag(self, name, revid):
         # FIXME: Not supported yet, should do a push of a new ref
@@ -287,7 +305,7 @@ class RemoteGitBranch(GitBranch):
 
     def __init__(self, bzrdir, repository, name, lockfiles):
         self._ref = None
-        super(RemoteGitBranch, self).__init__(bzrdir, repository, name, 
+        super(RemoteGitBranch, self).__init__(bzrdir, repository, name,
                 lockfiles)
 
     def revision_history(self):
@@ -296,6 +314,14 @@ class RemoteGitBranch(GitBranch):
     def last_revision(self):
         return self.mapping.revision_id_foreign_to_bzr(self.head)
 
+    def _get_config(self):
+        class EmptyConfig(object):
+
+            def _get_configobj(self):
+                return config.ConfigObj()
+
+        return EmptyConfig()
+
     @property
     def head(self):
         if self._ref is not None:
@@ -309,7 +335,7 @@ class RemoteGitBranch(GitBranch):
     def _synchronize_history(self, destination, revision_id):
         """See Branch._synchronize_history()."""
         destination.generate_revision_history(self.last_revision())
- 
+
     def get_push_location(self):
         return None
 
